Benchmarks
---

*Dislcaimer: remember benchmarks are generally fraught with issues and their
legitimacy depends on a multitude of factors.*

I've used basic, but seemingly nice, [`pytest-benchmark`](https://pytest-benchmark.readthedocs.io/en/stable/usage.html)
to produce the following benchmarks on a fairly modern machine.

If you'd like to run these yourself, you'll need whatever according system 
dependency, if any, required by the specific algorithm as well as the Python
lib installed. Can install all via `pip install -r benchmark-requirements.txt`. 

The resulting output from benchmarks indicate what file and lib was used, (`cramjam` vs `gzip`)
`test_gzip[urls.10K-cramjam]` indicates `cramjam` and `benchmarks/data/urls.10K` file was used during the gzip benchmark.


Special performance notes:
---
In general, if the length of the output from de/compression is known, for any variant,  
you can provide `output_len=<<some int>>` to get a 1-4x performance increase. This
is allows us to avoid double allocation. 

Furthermore, you can provide `output_len` equal to the "at most" length when 
de/compressing `bytearray` objects; as these can be resized after the de/compression 
is finished and actual size is known for basically the same performance. This is not
true for `bytes` objects, as we cannot resize after de/compression and the result
will be trialing null bytes.

For snappy used in conjunction with `bytearray`s, this is only midly helpful as
we automatically estimate the buffer size and can resize the `bytearray` after.


--- 

#### Gzip

`make bench-gzip`

(spoiler alert, although cramjam is generally faster than the builtin gzip module, you should pretty much always use [igzip](https://github.com/pycompression/python-isal) when possible.)

```bash 
------------------------------------------------------------------------------------------------------------------ benchmark: 42 tests -------------------------------------------------------------------------------------------------------------------
Name (time in us)                                           Min                       Max                      Mean                 StdDev                    Median                    IQR            Outliers          OPS            Rounds  Iterations
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_gzip[Mark.Twain-Tom.Sawyer.txt-cramjam]           191.8590 (3.44)           459.5510 (2.67)           232.3503 (3.49)         42.3918 (4.45)           216.2325 (3.43)         23.9335 (3.31)      505;534   4,303.8455 (0.29)       3484           1
test_gzip[Mark.Twain-Tom.Sawyer.txt-gzip]              230.9320 (4.14)         1,348.3770 (7.83)           278.0395 (4.17)         56.0642 (5.89)           259.9420 (4.12)         29.7642 (4.11)      410;435   3,596.6108 (0.24)       2899           1
test_gzip[Mark.Twain-Tom.Sawyer.txt-isal]               55.8100 (1.0)            172.2580 (1.0)             66.6014 (1.0)           9.5214 (1.0)             63.0650 (1.0)           7.2408 (1.0)       854;539  15,014.6976 (1.0)        6917           1
test_gzip[alice29.txt-cramjam]                       2,289.4180 (41.02)        3,586.7580 (20.82)        2,531.1701 (38.00)       205.3819 (21.57)        2,481.5745 (39.35)       106.6280 (14.73)       32;33     395.0742 (0.03)        320           1
test_gzip[alice29.txt-gzip]                          2,746.6730 (49.21)        5,784.2850 (33.58)        3,128.5895 (46.97)       352.9359 (37.07)        3,018.4270 (47.86)       258.2175 (35.66)       39;29     319.6329 (0.02)        320           1
test_gzip[alice29.txt-isal]                            630.2320 (11.29)        2,712.9960 (15.75)          692.1121 (10.39)        92.0883 (9.67)           673.7130 (10.68)        25.5560 (3.53)       70;159   1,444.8527 (0.10)       1288           1
test_gzip[asyoulik.txt-cramjam]                      2,031.5850 (36.40)        3,478.0610 (20.19)        2,246.2740 (33.73)       215.1429 (22.60)        2,193.3480 (34.78)       107.9130 (14.90)       30;39     445.1817 (0.03)        387           1
test_gzip[asyoulik.txt-gzip]                         2,336.2790 (41.86)        3,485.5790 (20.23)        2,547.9955 (38.26)       173.5410 (18.23)        2,504.1120 (39.71)       113.7950 (15.72)       34;29     392.4654 (0.03)        354           1
test_gzip[asyoulik.txt-isal]                           514.5240 (9.22)         1,400.8010 (8.13)           599.2456 (9.00)         73.4877 (7.72)           574.7005 (9.11)         71.0005 (9.81)       190;82   1,668.7649 (0.11)       1256           1
test_gzip[fifty-four-mb-random-cramjam]          1,211,255.6200 (>1000.0)  1,257,472.9720 (>1000.0)  1,237,629.0870 (>1000.0)  17,775.0203 (>1000.0)  1,241,997.1640 (>1000.0)  24,676.1108 (>1000.0)       2;0       0.8080 (0.00)          5           1
test_gzip[fifty-four-mb-random-gzip]             1,095,648.4620 (>1000.0)  1,110,366.0260 (>1000.0)  1,103,719.8038 (>1000.0)   6,105.5552 (641.24)   1,102,582.1870 (>1000.0)   9,973.5320 (>1000.0)       2;0       0.9060 (0.00)          5           1
test_gzip[fifty-four-mb-random-isal]               168,541.5770 (>1000.0)    212,197.5160 (>1000.0)    181,629.0707 (>1000.0)  15,576.1286 (>1000.0)    177,291.2230 (>1000.0)   7,280.1230 (>1000.0)       1;1       5.5057 (0.00)          6           1
test_gzip[fifty-four-mb-repeating-cramjam]         134,944.4950 (>1000.0)    181,172.1680 (>1000.0)    153,997.7280 (>1000.0)  19,735.0671 (>1000.0)    142,841.9530 (>1000.0)  36,666.9240 (>1000.0)       2;0       6.4936 (0.00)          7           1
test_gzip[fifty-four-mb-repeating-gzip]            126,086.2200 (>1000.0)    144,891.1680 (841.13)     132,237.1890 (>1000.0)   6,698.9614 (703.57)     129,553.7330 (>1000.0)   9,267.9080 (>1000.0)       2;0       7.5622 (0.00)          8           1
test_gzip[fifty-four-mb-repeating-isal]            104,930.9420 (>1000.0)    110,588.2970 (641.99)     107,397.2519 (>1000.0)   1,882.4948 (197.71)     107,018.3145 (>1000.0)   3,244.9360 (448.15)        4;0       9.3112 (0.00)         10           1
test_gzip[fireworks.jpeg-cramjam]                    2,272.0720 (40.71)        3,841.4350 (22.30)        2,544.0224 (38.20)       226.1681 (23.75)        2,484.9960 (39.40)       189.9015 (26.23)       30;18     393.0783 (0.03)        344           1
test_gzip[fireworks.jpeg-gzip]                       2,033.2690 (36.43)        3,293.9500 (19.12)        2,261.8599 (33.96)       128.4785 (13.49)        2,230.3385 (35.37)        83.7080 (11.56)       68;38     442.1140 (0.03)        410           1
test_gzip[fireworks.jpeg-isal]                         557.8950 (10.00)          922.6780 (5.36)           639.2987 (9.60)         53.6616 (5.64)           638.0545 (10.12)        79.6670 (11.00)      318;19   1,564.2139 (0.10)       1058           1
test_gzip[geo.protodata-cramjam]                       543.6170 (9.74)         3,331.8180 (19.34)          683.1062 (10.26)       164.7324 (17.30)          653.2410 (10.36)        73.8828 (10.20)     137;148   1,463.9011 (0.10)       1507           1
test_gzip[geo.protodata-gzip]                          746.4350 (13.37)        3,485.2950 (20.23)          832.4478 (12.50)       108.8803 (11.44)          815.1780 (12.93)        51.7680 (7.15)        48;66   1,201.2765 (0.08)       1016           1
test_gzip[geo.protodata-isal]                          325.9980 (5.84)         1,716.9900 (9.97)           360.5302 (5.41)         43.9673 (4.62)           350.1455 (5.55)         18.2805 (2.52)      183;239   2,773.6924 (0.18)       2228           1
test_gzip[html-cramjam]                                559.0320 (10.02)        1,007.0500 (5.85)           652.3441 (9.79)         62.2105 (6.53)           644.8590 (10.23)        58.2925 (8.05)        95;39   1,532.9332 (0.10)        800           1
test_gzip[html-gzip]                                   733.5640 (13.14)        1,239.4010 (7.20)           814.8288 (12.23)        61.7839 (6.49)           795.7915 (12.62)        60.2950 (8.33)       138;44   1,227.2517 (0.08)        912           1
test_gzip[html-isal]                                   293.4400 (5.26)         1,077.9470 (6.26)           320.2257 (4.81)         31.3367 (3.29)           312.7960 (4.96)         17.4120 (2.40)      113;117   3,122.7978 (0.21)       1680           1
test_gzip[html_x_4-cramjam]                          2,680.6610 (48.03)        5,121.8420 (29.73)        2,949.2002 (44.28)       183.4686 (19.27)        2,922.4515 (46.34)        90.8240 (12.54)       31;29     339.0750 (0.02)        326           1
test_gzip[html_x_4-gzip]                             3,140.6020 (56.27)        6,755.7770 (39.22)        3,438.3153 (51.63)       334.5340 (35.13)        3,389.2870 (53.74)       178.4918 (24.65)       13;16     290.8401 (0.02)        249           1
test_gzip[html_x_4-isal]                             1,145.9080 (20.53)        5,369.5510 (31.17)        1,333.4371 (20.02)       263.7536 (27.70)        1,255.0020 (19.90)       128.4425 (17.74)       75;95     749.9417 (0.05)        693           1
test_gzip[kppkn.gtb-cramjam]                         2,272.2560 (40.71)        3,975.8810 (23.08)        2,460.3480 (36.94)       138.0943 (14.50)        2,439.8625 (38.69)        80.1670 (11.07)       35;31     406.4466 (0.03)        378           1
test_gzip[kppkn.gtb-gzip]                            2,737.3670 (49.05)        4,541.3070 (26.36)        2,997.0626 (45.00)       208.5254 (21.90)        2,938.9400 (46.60)       126.3835 (17.45)       36;29     333.6600 (0.02)        327           1
test_gzip[kppkn.gtb-isal]                              620.2470 (11.11)        1,438.9580 (8.35)           694.6918 (10.43)        77.9132 (8.18)           664.6515 (10.54)        36.2180 (5.00)      118;136   1,439.4872 (0.10)       1038           1
test_gzip[lcet10.txt-cramjam]                        6,515.5340 (116.74)      10,480.1160 (60.84)        7,804.9459 (117.19)    1,293.0755 (135.81)       7,175.8955 (113.79)    2,331.5930 (322.01)       34;0     128.1239 (0.01)        138           1
test_gzip[lcet10.txt-gzip]                           7,590.8110 (136.01)      14,044.3360 (81.53)        8,259.7372 (124.02)      803.4726 (84.39)        8,047.0785 (127.60)      407.9500 (56.34)         9;9     121.0692 (0.01)        120           1
test_gzip[lcet10.txt-isal]                           1,798.1930 (32.22)        3,801.2430 (22.07)        2,001.5485 (30.05)       216.8714 (22.78)        1,937.9530 (30.73)       107.5470 (14.85)       40;47     499.6132 (0.03)        390           1
test_gzip[paper-100k.pdf-cramjam]                    1,574.2830 (28.21)        4,883.8400 (28.35)        1,773.2297 (26.62)       189.1485 (19.87)        1,735.9090 (27.53)        65.1753 (9.00)        27;45     563.9427 (0.04)        541           1
test_gzip[paper-100k.pdf-gzip]                       1,876.8750 (33.63)        4,374.8760 (25.40)        2,054.1447 (30.84)       168.2990 (17.68)        2,019.5360 (32.02)        87.6453 (12.10)       30;33     486.8206 (0.03)        443           1
test_gzip[paper-100k.pdf-isal]                         421.2930 (7.55)         3,587.9140 (20.83)          483.5627 (7.26)        113.1682 (11.89)          464.7290 (7.37)         43.4210 (6.00)        50;90   2,067.9842 (0.14)       1359           1
test_gzip[plrabn12.txt-cramjam]                      8,824.4760 (158.12)      12,236.2950 (71.03)        9,246.9970 (138.84)      499.8499 (52.50)        9,109.3310 (144.44)      171.8368 (23.73)         7;7     108.1432 (0.01)         75           1
test_gzip[plrabn12.txt-gzip]                        10,067.7340 (180.39)      14,851.2830 (86.22)       10,705.5403 (160.74)      849.3527 (89.20)       10,398.4875 (164.89)      414.7850 (57.28)         9;9      93.4096 (0.01)         90           1
test_gzip[plrabn12.txt-isal]                         1,997.3020 (35.79)        3,128.8200 (18.16)        2,267.3735 (34.04)       257.9910 (27.10)        2,136.7485 (33.88)       234.7265 (32.42)       74;52     441.0389 (0.03)        396           1
test_gzip[urls.10K-cramjam]                          9,862.8800 (176.72)      14,844.7900 (86.18)       10,724.2182 (161.02)    1,308.2085 (137.40)      10,164.5780 (161.18)      576.2437 (79.58)       10;14      93.2469 (0.01)         91           1
test_gzip[urls.10K-gzip]                            10,704.3820 (191.80)      13,596.6140 (78.93)       11,235.3842 (168.70)      542.1322 (56.94)       11,038.5890 (175.04)      338.8595 (46.80)         9;9      89.0045 (0.01)         81           1
test_gzip[urls.10K-isal]                             2,942.9670 (52.73)        7,301.6880 (42.39)        3,212.3236 (48.23)       316.5106 (33.24)        3,136.4275 (49.73)       149.9830 (20.71)       21;28     311.3011 (0.02)        276           1
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```

#### Snappy

There are two types here, `framed` and `raw`; the recommended one being snappy's `framed` format.

`make bench-snappy-framed`

```bash
------------------------------------------------------------------------------------------------------------------------- benchmark: 28 tests -------------------------------------------------------------------------------------------------------------------------
Name (time in us)                                                    Min                       Max                      Mean                    StdDev                    Median                     IQR            Outliers          OPS            Rounds  Iterations
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_snappy_framed[Mark.Twain-Tom.Sawyer.txt-cramjam]            67.8750 (1.0)            184.4690 (1.0)             69.9458 (1.0)              5.8189 (1.0)             68.5219 (1.0)            0.4220 (1.72)      394;606  14,296.7757 (1.0)        5157           1
test_snappy_framed[Mark.Twain-Tom.Sawyer.txt-snappy]            127.5920 (1.88)           321.8240 (1.74)           131.6162 (1.88)             9.4019 (1.62)           128.8950 (1.88)           0.7390 (3.01)      407;905   7,597.8511 (0.53)       5234           1
test_snappy_framed[alice29.txt-cramjam]                         676.6730 (9.97)           965.0890 (5.23)           693.1161 (9.91)            27.2101 (4.68)           679.8870 (9.92)          20.7548 (84.45)      143;90   1,442.7596 (0.10)       1423           1
test_snappy_framed[alice29.txt-snappy]                        1,362.6880 (20.08)        1,884.4430 (10.22)        1,394.3984 (19.94)           43.0275 (7.39)         1,385.3500 (20.22)         39.8223 (162.04)      62;37     717.1552 (0.05)        679           1
test_snappy_framed[asyoulik.txt-cramjam]                        593.6730 (8.75)           886.0190 (4.80)           607.4296 (8.68)            24.3360 (4.18)           596.2565 (8.70)          19.8330 (80.70)      174;85   1,646.2814 (0.12)       1612           1
test_snappy_framed[asyoulik.txt-snappy]                       1,154.8221 (17.01)        1,474.2750 (7.99)         1,179.9538 (16.87)           33.2390 (5.71)         1,169.8285 (17.07)         33.0758 (134.59)      86;34     847.4908 (0.06)        814           1
test_snappy_framed[fifty-four-mb-random-cramjam]            106,125.5330 (>1000.0)    122,712.4280 (665.22)     113,907.3788 (>1000.0)      4,144.4500 (712.24)     113,855.9120 (>1000.0)    2,535.2445 (>1000.0)       3;2       8.7791 (0.00)         11           1
test_snappy_framed[fifty-four-mb-random-snappy]           5,736,722.0209 (>1000.0)  8,884,037.3160 (>1000.0)  8,029,947.1742 (>1000.0)  1,294,217.3836 (>1000.0)  8,468,064.0850 (>1000.0)  931,786.2789 (>1000.0)       1;1       0.1245 (0.00)          5           1
test_snappy_framed[fifty-four-mb-repeating-cramjam]          60,967.8230 (898.24)      82,541.3500 (447.45)      68,490.7448 (979.20)       8,907.2626 (>1000.0)     62,633.5774 (914.07)    16,801.8445 (>1000.0)       4;0      14.6005 (0.00)         12           1
test_snappy_framed[fifty-four-mb-repeating-snappy]          416,213.7700 (>1000.0)    445,269.2589 (>1000.0)    425,080.6548 (>1000.0)     12,411.2216 (>1000.0)    418,530.8550 (>1000.0)   16,490.2027 (>1000.0)       1;0       2.3525 (0.00)          5           1
test_snappy_framed[fireworks.jpeg-cramjam]                       90.7030 (1.34)           265.3861 (1.44)            93.6783 (1.34)             6.9734 (1.20)            91.9680 (1.34)           0.3370 (1.37)     696;1196  10,674.8307 (0.75)       8521           1
test_snappy_framed[fireworks.jpeg-snappy]                       634.0879 (9.34)           905.7500 (4.91)           653.3697 (9.34)            24.7685 (4.26)           643.7055 (9.39)          14.7215 (59.90)     151;145   1,530.5271 (0.11)       1472           1
test_snappy_framed[geo.protodata-cramjam]                       204.8140 (3.02)           484.7000 (2.63)           210.1287 (3.00)            12.0245 (2.07)           206.3761 (3.01)           0.9029 (3.67)      410;868   4,758.9888 (0.33)       4299           1
test_snappy_framed[geo.protodata-snappy]                        727.0420 (10.71)          979.4390 (5.31)           744.5916 (10.65)           25.8275 (4.44)           732.3179 (10.69)         22.5262 (91.66)      155;72   1,343.0181 (0.09)       1299           1
test_snappy_framed[html-cramjam]                                211.0000 (3.11)           400.2160 (2.17)           216.8802 (3.10)            12.6234 (2.17)           212.4980 (3.10)           0.7945 (3.23)      435;805   4,610.8402 (0.32)       3736           1
test_snappy_framed[html-snappy]                                 662.6330 (9.76)         1,020.1860 (5.53)           695.4484 (9.94)            29.1009 (5.00)           686.1100 (10.01)         23.4039 (95.23)      345;76   1,437.9211 (0.10)       1350           1
test_snappy_framed[html_x_4-cramjam]                            814.7360 (12.00)        1,156.0221 (6.27)           833.7065 (11.92)           34.9787 (6.01)           819.8969 (11.97)         21.4603 (87.32)       81;76   1,199.4629 (0.08)       1083           1
test_snappy_framed[html_x_4-snappy]                           2,654.4550 (39.11)        3,212.9940 (17.42)        2,729.8276 (39.03)           92.7821 (15.94)        2,701.6510 (39.43)         73.2925 (298.24)      31;25     366.3235 (0.03)        336           1
test_snappy_framed[kppkn.gtb-cramjam]                           582.3770 (8.58)           868.2209 (4.71)           596.1833 (8.52)            23.0826 (3.97)           584.8820 (8.54)          19.5719 (79.64)      194;81   1,677.3366 (0.12)       1515           1
test_snappy_framed[kppkn.gtb-snappy]                          1,413.1500 (20.82)        1,795.0049 (9.73)         1,441.8433 (20.61)           38.8178 (6.67)         1,435.4640 (20.95)         34.1205 (138.84)      54;30     693.5566 (0.05)        603           1
test_snappy_framed[lcet10.txt-cramjam]                        1,766.2169 (26.02)        2,194.9910 (11.90)        1,809.2666 (25.87)           52.9258 (9.10)         1,795.1445 (26.20)         50.4309 (205.21)      42;24     552.7101 (0.04)        526           1
test_snappy_framed[lcet10.txt-snappy]                         3,776.0150 (55.63)        4,416.6800 (23.94)        3,865.1326 (55.26)           90.4870 (15.55)        3,843.4530 (56.09)         87.1853 (354.77)      22;12     258.7233 (0.02)        235           1
test_snappy_framed[paper-100k.pdf-cramjam]                       84.9301 (1.25)           217.1260 (1.18)            87.1440 (1.25)             6.4667 (1.11)            85.6020 (1.25)           0.2458 (1.0)      593;1125  11,475.2531 (0.80)       8555           1
test_snappy_framed[paper-100k.pdf-snappy]                       533.0379 (7.85)           845.9381 (4.59)           546.8427 (7.82)            24.6972 (4.24)           538.5000 (7.86)          16.9175 (68.84)     144;115   1,828.6796 (0.13)       1636           1
test_snappy_framed[plrabn12.txt-cramjam]                      2,386.8741 (35.17)        3,894.9240 (21.11)        2,475.3461 (35.39)          169.0649 (29.05)        2,435.1560 (35.54)         63.3658 (257.84)      20;30     403.9839 (0.03)        339           1
test_snappy_framed[plrabn12.txt-snappy]                       5,019.8400 (73.96)        5,844.5639 (31.68)        5,131.9534 (73.37)          112.6590 (19.36)        5,106.9070 (74.53)         89.4035 (363.79)      15;13     194.8576 (0.01)        188           1
test_snappy_framed[urls.10K-cramjam]                          2,149.3380 (31.67)        2,757.3260 (14.95)        2,204.8847 (31.52)           69.6202 (11.96)        2,185.4395 (31.89)         61.6229 (250.75)      30;15     453.5384 (0.03)        406           1
test_snappy_framed[urls.10K-snappy]                           5,894.0420 (86.84)        6,559.6290 (35.56)        6,029.6206 (86.20)          120.3123 (20.68)        5,994.6314 (87.48)         84.0485 (342.00)      26;18     165.8479 (0.01)        164           1
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```

`make bench-snappy-raw`

```bash
------------------------------------------------------------------------------------------------------------ benchmark: 28 tests -------------------------------------------------------------------------------------------------------------
Name (time in us)                                              Min                    Max                   Mean              StdDev                 Median                 IQR            Outliers          OPS            Rounds  Iterations
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_snappy_raw[Mark.Twain-Tom.Sawyer.txt-cramjam]         51.9970 (3.43)        110.7800 (2.43)         53.6142 (3.37)       3.9642 (2.32)         52.7401 (3.38)       0.3098 (2.50)      432;630  18,651.7875 (0.30)       7791           1
test_snappy_raw[Mark.Twain-Tom.Sawyer.txt-snappy]          52.7010 (3.47)        122.6700 (2.69)         54.9113 (3.45)       4.9091 (2.88)         53.6260 (3.44)       0.4690 (3.78)     768;1676  18,211.1850 (0.29)      11113           1
test_snappy_raw[alice29.txt-cramjam]                      599.3680 (39.51)       895.2310 (19.61)       613.8907 (38.59)     22.1703 (13.00)       603.5195 (38.70)     20.4269 (164.76)     133;54   1,628.9544 (0.03)       1158           1
test_snappy_raw[alice29.txt-snappy]                       600.2350 (39.57)       775.9969 (17.00)       613.0591 (38.54)     17.8929 (10.49)       603.6960 (38.71)     19.4444 (156.83)     231;68   1,631.1640 (0.03)       1631           1
test_snappy_raw[asyoulik.txt-cramjam]                     532.1550 (35.08)       831.1180 (18.21)       545.4919 (34.29)     21.3844 (12.54)       535.0631 (34.31)     19.8718 (160.28)     167;63   1,833.2078 (0.03)       1307           1
test_snappy_raw[asyoulik.txt-snappy]                      532.5220 (35.10)       800.6040 (17.54)       544.5727 (34.24)     18.2074 (10.67)       536.1935 (34.38)     12.6080 (101.69)    219;197   1,836.3022 (0.03)       1830           1
test_snappy_raw[fifty-four-mb-random-cramjam]          37,541.5270 (>1000.0)  38,867.6571 (851.52)   37,919.1172 (>1000.0)  307.6292 (180.35)   37,829.9700 (>1000.0)  311.4055 (>1000.0)       6;2      26.3719 (0.00)         27           1
test_snappy_raw[fifty-four-mb-random-snappy]           54,326.6139 (>1000.0)  55,845.7799 (>1000.0)  54,884.3537 (>1000.0)  412.7079 (241.95)   54,832.6219 (>1000.0)  475.0847 (>1000.0)       6;1      18.2201 (0.00)         19           1
test_snappy_raw[fifty-four-mb-repeating-cramjam]       17,394.3370 (>1000.0)  18,066.0550 (395.79)   17,612.4400 (>1000.0)  163.3287 (95.75)    17,561.4910 (>1000.0)  237.1181 (>1000.0)      11;0      56.7780 (0.00)         29           1
test_snappy_raw[fifty-four-mb-repeating-snappy]        31,430.8960 (>1000.0)  32,698.4840 (716.36)   31,772.2997 (>1000.0)  271.8508 (159.38)   31,746.9710 (>1000.0)  266.7012 (>1000.0)       4;2      31.4740 (0.00)         31           1
test_snappy_raw[fireworks.jpeg-cramjam]                    25.3069 (1.67)         59.5120 (1.30)         26.2138 (1.65)       2.6992 (1.58)         25.5830 (1.64)       0.1240 (1.0)       349;901  38,147.7836 (0.61)       7113           1
test_snappy_raw[fireworks.jpeg-snappy]                     15.1701 (1.0)          45.6451 (1.0)          15.9064 (1.0)        1.7057 (1.0)          15.5960 (1.0)        0.1411 (1.14)     988;5393  62,867.9262 (1.0)       35170           1
test_snappy_raw[geo.protodata-cramjam]                    155.7680 (10.27)       250.6210 (5.49)        160.3773 (10.08)      9.2061 (5.40)        157.2450 (10.08)      0.8419 (6.79)      356;768   6,235.2982 (0.10)       3922           1
test_snappy_raw[geo.protodata-snappy]                     142.9900 (9.43)        266.7190 (5.84)        148.3392 (9.33)       8.0790 (4.74)        145.8429 (9.35)       2.2233 (17.93)     471;684   6,741.3054 (0.11)       5261           1
test_snappy_raw[html-cramjam]                             165.5300 (10.91)       266.9621 (5.85)        169.8480 (10.68)      9.1233 (5.35)        166.7660 (10.69)      0.6332 (5.11)      348;702   5,887.6187 (0.09)       3877           1
test_snappy_raw[html-snappy]                              156.3310 (10.31)       247.4729 (5.42)        160.6741 (10.10)      8.7547 (5.13)        157.9490 (10.13)      0.7500 (6.05)      480;904   6,223.7766 (0.10)       5697           1
test_snappy_raw[html_x_4-cramjam]                         660.6469 (43.55)     1,195.4820 (26.19)       678.1065 (42.63)     39.0818 (22.91)       664.1049 (42.58)     20.3864 (164.43)      42;47   1,474.6945 (0.02)       1035           1
test_snappy_raw[html_x_4-snappy]                          633.0810 (41.73)       917.3179 (20.10)       648.8233 (40.79)     21.6437 (12.69)       638.4600 (40.94)     20.0127 (161.42)     193;63   1,541.2517 (0.02)       1451           1
test_snappy_raw[kppkn.gtb-cramjam]                        499.2380 (32.91)       708.5471 (15.52)       510.3782 (32.09)     18.7145 (10.97)       501.3411 (32.15)     14.9945 (120.94)    175;131   1,959.3315 (0.03)       1355           1
test_snappy_raw[kppkn.gtb-snappy]                         503.4071 (33.18)       733.6430 (16.07)       514.8920 (32.37)     18.8445 (11.05)       505.8900 (32.44)     14.7057 (118.61)    223;174   1,942.1549 (0.03)       1907           1
test_snappy_raw[lcet10.txt-cramjam]                     1,585.7200 (104.53)    1,902.8189 (41.69)     1,620.1644 (101.86)    42.2322 (24.76)     1,608.8235 (103.16)    43.0678 (347.37)      57;18     617.2213 (0.01)        486           1
test_snappy_raw[lcet10.txt-snappy]                      1,590.4551 (104.84)    2,116.2720 (46.36)     1,627.2067 (102.30)    47.9340 (28.10)     1,616.7820 (103.67)    41.8220 (337.32)      48;22     614.5501 (0.01)        534           1
test_snappy_raw[paper-100k.pdf-cramjam]                    28.1520 (1.86)         69.1239 (1.51)         29.0821 (1.83)       2.6677 (1.56)         28.4630 (1.83)       0.1448 (1.17)     344;1243  34,385.3648 (0.55)       7894           1
test_snappy_raw[paper-100k.pdf-snappy]                     20.3360 (1.34)         85.0500 (1.86)         21.3502 (1.34)       2.1718 (1.27)         20.9491 (1.34)       0.4959 (4.00)     930;1321  46,838.0012 (0.75)      25100           1
test_snappy_raw[plrabn12.txt-cramjam]                   2,169.4830 (143.01)    4,583.7279 (100.42)    2,236.1366 (140.58)   180.7923 (105.99)    2,212.1780 (141.84)    60.5080 (488.04)       7;10     447.1999 (0.01)        318           1
test_snappy_raw[plrabn12.txt-snappy]                    2,187.1999 (144.18)    2,717.8620 (59.54)     2,246.2940 (141.22)    67.8218 (39.76)     2,230.6465 (143.03)    47.7184 (384.88)      38;30     445.1777 (0.01)        392           1
test_snappy_raw[urls.10K-cramjam]                       1,852.2060 (122.10)    2,464.6381 (54.00)     1,899.2503 (119.40)    65.4195 (38.35)     1,882.6990 (120.72)    52.5715 (424.02)      31;18     526.5235 (0.01)        424           1
test_snappy_raw[urls.10K-snappy]                        1,814.2400 (119.59)    2,366.7030 (51.85)     1,855.7408 (116.67)    63.5185 (37.24)     1,840.4050 (118.00)    45.1091 (363.83)      25;20     538.8684 (0.01)        438           1
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```


##### Benchmarks for using `compress_into` and `decompress_into`
All variants implement a `compress_into` and `decompress_into`
This is where, if you have a Python numpy array pre-allocated to the size of the
de/compress data output, we can place the bytes directly into that buffer.

Again, since basically no variants implement similar functionality as we saw in the Gzip `output_len`
benchmarks, this benchmark is specific to `cramjam`

```bash
--------------------------------------------------------------------------------------------------------------------------- benchmark: 28 tests ----------------------------------------------------------------------------------------------------------------------------
Name (time in us)                                                                           Min                    Max                   Mean              StdDev                 Median                 IQR            Outliers           OPS            Rounds  Iterations
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_cramjam_snappy_de_compress_into[Mark.Twain-Tom.Sawyer.txt-compress_into]           40.5740 (6.10)         80.9841 (4.98)         42.0519 (6.02)       3.3541 (3.54)         41.1549 (5.99)       0.3109 (4.71)    1006;2779   23,780.1522 (0.17)      17961           1
test_cramjam_snappy_de_compress_into[Mark.Twain-Tom.Sawyer.txt-decompress_into]          6.7360 (1.01)         21.8459 (1.34)          6.9848 (1.0)        0.9464 (1.0)           6.8730 (1.0)        0.0660 (1.0)       122;700  143,168.7285 (1.0)        9684           1
test_cramjam_snappy_de_compress_into[alice29.txt-compress_into]                        461.2940 (69.40)       669.6520 (41.18)       473.9482 (67.85)     18.6320 (19.69)       464.7850 (67.62)     15.4326 (233.80)    240;175    2,109.9353 (0.01)       1992           1
test_cramjam_snappy_de_compress_into[alice29.txt-decompress_into]                        6.7560 (1.02)         22.4550 (1.38)          7.1457 (1.02)       0.9911 (1.05)          7.0620 (1.03)       0.1279 (1.94)       51;203  139,943.6715 (0.98)       3866           1
test_cramjam_snappy_de_compress_into[asyoulik.txt-compress_into]                       408.1880 (61.41)       628.2150 (38.64)       420.7408 (60.24)     17.0570 (18.02)       411.6645 (59.90)     19.7020 (298.48)     301;60    2,376.7602 (0.02)       2338           1
test_cramjam_snappy_de_compress_into[asyoulik.txt-decompress_into]                       6.6471 (1.0)          35.4199 (2.18)          7.1268 (1.02)       1.2031 (1.27)          7.0110 (1.02)       0.0992 (1.50)       86;954  140,315.1785 (0.98)       5545           1
test_cramjam_snappy_de_compress_into[fifty-four-mb-random-compress_into]            21,292.6060 (>1000.0)  22,134.7660 (>1000.0)  21,556.6603 (>1000.0)  195.2096 (206.28)   21,524.6500 (>1000.0)  282.1560 (>1000.0)       7;1       46.3894 (0.00)         26           1
test_cramjam_snappy_de_compress_into[fifty-four-mb-random-decompress_into]               6.8980 (1.04)         17.6111 (1.08)          7.4460 (1.07)       1.8980 (2.01)          7.0570 (1.03)       0.2597 (3.93)          1;2  134,299.5668 (0.94)         31           1
test_cramjam_snappy_de_compress_into[fifty-four-mb-repeating-compress_into]         14,725.2050 (>1000.0)  15,681.2500 (964.41)   14,952.5908 (>1000.0)  172.2539 (182.02)   14,909.9910 (>1000.0)  154.4059 (>1000.0)       8;3       66.8780 (0.00)         62           1
test_cramjam_snappy_de_compress_into[fifty-four-mb-repeating-decompress_into]            6.8520 (1.03)         16.2600 (1.0)           7.4351 (1.06)       1.7599 (1.86)          7.0125 (1.02)       0.1526 (2.31)          1;5  134,496.9292 (0.94)         28           1
test_cramjam_snappy_de_compress_into[fireworks.jpeg-compress_into]                      45.8390 (6.90)         94.1610 (5.79)         47.7454 (6.84)       3.5484 (3.75)         46.8700 (6.82)       0.8349 (12.65)   1167;2927   20,944.4108 (0.15)      19418           1
test_cramjam_snappy_de_compress_into[fireworks.jpeg-decompress_into]                     6.7510 (1.02)         41.1160 (2.53)          7.1202 (1.02)       1.1970 (1.26)          6.9160 (1.01)       0.1888 (2.86)      515;974  140,445.6319 (0.98)      25688           1
test_cramjam_snappy_de_compress_into[geo.protodata-compress_into]                      120.1930 (18.08)       224.7400 (13.82)       124.6344 (17.84)      7.8673 (8.31)        122.0175 (17.75)      1.2110 (18.35)    666;1483    8,023.4694 (0.06)       7554           1
test_cramjam_snappy_de_compress_into[geo.protodata-decompress_into]                      6.7500 (1.02)         28.4720 (1.75)          7.2207 (1.03)       1.2935 (1.37)          7.0831 (1.03)       0.2290 (3.47)      260;297  138,490.1876 (0.97)      11741           1
test_cramjam_snappy_de_compress_into[html-compress_into]                               128.7600 (19.37)       240.4600 (14.79)       133.1894 (19.07)      8.4420 (8.92)        130.3609 (18.97)      0.9710 (14.71)    619;1486    7,508.1029 (0.05)       7215           1
test_cramjam_snappy_de_compress_into[html-decompress_into]                               6.7130 (1.01)         29.0590 (1.79)          7.3321 (1.05)       1.5681 (1.66)          6.9850 (1.02)       0.2070 (3.14)      878;946  136,386.3798 (0.95)      12593           1
test_cramjam_snappy_de_compress_into[html_x_4-compress_into]                           510.7940 (76.84)       783.9740 (48.21)       525.5117 (75.24)     21.4496 (22.67)       514.8470 (74.91)     20.4052 (309.13)     227;82    1,902.9072 (0.01)       1807           1
test_cramjam_snappy_de_compress_into[html_x_4-decompress_into]                           6.7271 (1.01)         26.0500 (1.60)          7.2545 (1.04)       1.3682 (1.45)          6.9321 (1.01)       0.3700 (5.60)       89;281  137,844.8162 (0.96)       3292           1
test_cramjam_snappy_de_compress_into[kppkn.gtb-compress_into]                          360.5250 (54.24)       599.6680 (36.88)       371.5869 (53.20)     15.3924 (16.26)       363.2680 (52.85)     20.0116 (303.17)     306;42    2,691.1603 (0.02)       2497           1
test_cramjam_snappy_de_compress_into[kppkn.gtb-decompress_into]                          6.7559 (1.02)         28.5150 (1.75)          7.2983 (1.04)       1.3938 (1.47)          7.1101 (1.03)       0.2340 (3.54)      129;254  137,018.7128 (0.96)       4798           1
test_cramjam_snappy_de_compress_into[lcet10.txt-compress_into]                       1,220.3240 (183.59)    1,442.1020 (88.69)     1,252.7170 (179.35)    34.3613 (36.31)     1,244.1790 (181.02)    44.0781 (667.77)     138;22      798.2649 (0.01)        814           1
test_cramjam_snappy_de_compress_into[lcet10.txt-decompress_into]                         6.7160 (1.01)         27.3180 (1.68)          7.2492 (1.04)       1.3898 (1.47)          7.0255 (1.02)       0.3814 (5.78)       60;125  137,946.6539 (0.96)       1868           1
test_cramjam_snappy_de_compress_into[paper-100k.pdf-compress_into]                      39.6080 (5.96)         91.3910 (5.62)         41.3625 (5.92)       3.3869 (3.58)         40.6440 (5.91)       0.9619 (14.57)   1132;1662   24,176.4958 (0.17)      20478           1
test_cramjam_snappy_de_compress_into[paper-100k.pdf-decompress_into]                     6.7310 (1.01)         31.5481 (1.94)          7.1218 (1.02)       1.1942 (1.26)          6.9310 (1.01)       0.1870 (2.83)      440;725  140,414.1376 (0.98)      23261           1
test_cramjam_snappy_de_compress_into[plrabn12.txt-compress_into]                     1,651.1610 (248.40)    2,189.8210 (134.68)    1,689.8730 (241.94)    46.3566 (48.98)     1,677.2315 (244.03)    50.6585 (767.47)      69;14      591.7604 (0.00)        572           1
test_cramjam_snappy_de_compress_into[plrabn12.txt-decompress_into]                       6.6870 (1.01)         36.4761 (2.24)          7.1576 (1.02)       1.5466 (1.63)          6.8745 (1.00)       0.2451 (3.71)        44;53  139,711.1538 (0.98)       1228           1
test_cramjam_snappy_de_compress_into[urls.10K-compress_into]                         1,445.5540 (217.47)    1,788.2879 (109.98)    1,486.7689 (212.86)    41.1075 (43.44)     1,474.6580 (214.56)    52.1065 (789.40)      88;14      672.5995 (0.00)        636           1
test_cramjam_snappy_de_compress_into[urls.10K-decompress_into]                           6.7020 (1.01)         31.9510 (1.97)          7.3564 (1.05)       1.7521 (1.85)          6.8840 (1.00)       0.2775 (4.20)       85;117  135,935.3108 (0.95)       1117           1
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```

#### Lz4

`make bench-lz4`

```bash
------------------------------------------------------------------------------------------------------------------- benchmark: 28 tests -------------------------------------------------------------------------------------------------------------------
Name (time in us)                                             Min                       Max                      Mean                 StdDev                    Median                    IQR            Outliers         OPS            Rounds  Iterations
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_lz4[Mark.Twain-Tom.Sawyer.txt-cramjam]              245.3330 (1.0)          7,722.4160 (21.29)          364.4594 (1.05)        240.3840 (33.07)          368.7030 (1.06)         18.0650 (2.54)        3;265  2,743.7899 (0.95)       1155           1
test_lz4[Mark.Twain-Tom.Sawyer.txt-python-lz4]           335.9900 (1.37)           362.7009 (1.0)            347.2828 (1.0)           7.2700 (1.0)            346.6295 (1.0)           7.1260 (1.0)           2;1  2,879.4977 (1.0)          10           1
test_lz4[alice29.txt-cramjam]                          2,814.1720 (11.47)        4,936.4029 (13.61)        3,046.3559 (8.77)        379.7652 (52.24)        2,920.2300 (8.42)         70.8259 (9.94)        33;46    328.2611 (0.11)        338           1
test_lz4[alice29.txt-python-lz4]                       3,121.6950 (12.72)        5,633.2920 (15.53)        3,441.3794 (9.91)        478.2910 (65.79)        3,252.9815 (9.38)        140.1610 (19.67)       34;50    290.5812 (0.10)        300           1
test_lz4[asyoulik.txt-cramjam]                         2,314.5641 (9.43)         3,103.1220 (8.56)         2,382.4872 (6.86)         92.2261 (12.69)        2,359.6610 (6.81)         74.5750 (10.47)       29;18    419.7294 (0.15)        404           1
test_lz4[asyoulik.txt-python-lz4]                      2,633.2920 (10.73)        4,455.6119 (12.28)        2,722.9006 (7.84)        196.3372 (27.01)        2,682.5680 (7.74)         65.2385 (9.15)        10;21    367.2554 (0.13)        367           1
test_lz4[fifty-four-mb-random-cramjam]               757,487.6130 (>1000.0)    792,758.5000 (>1000.0)    772,111.8064 (>1000.0)  13,251.3858 (>1000.0)    769,733.5170 (>1000.0)  16,313.7843 (>1000.0)       2;0      1.2951 (0.00)          5           1
test_lz4[fifty-four-mb-random-python-lz4]          1,597,680.0489 (>1000.0)  1,637,720.9669 (>1000.0)  1,613,587.9904 (>1000.0)  17,438.2782 (>1000.0)  1,606,424.0520 (>1000.0)  29,249.6240 (>1000.0)       1;0      0.6197 (0.00)          5           1
test_lz4[fifty-four-mb-repeating-cramjam]            222,541.1070 (907.10)     237,449.5791 (654.67)     228,523.9746 (658.03)    6,592.2073 (906.77)     227,023.9190 (654.95)   11,598.6770 (>1000.0)       1;0      4.3759 (0.00)          5           1
test_lz4[fifty-four-mb-repeating-python-lz4]         161,025.9259 (656.36)     356,819.4229 (983.78)     214,613.3904 (617.98)   73,690.5136 (>1000.0)    186,189.4020 (537.14)   90,134.4944 (>1000.0)       1;0      4.6595 (0.00)          7           1
test_lz4[fireworks.jpeg-cramjam]                       1,556.7919 (6.35)         3,704.2680 (10.21)        1,665.9848 (4.80)        218.8115 (30.10)        1,613.6711 (4.66)         33.3220 (4.68)        26;59    600.2456 (0.21)        569           1
test_lz4[fireworks.jpeg-python-lz4]                    2,682.0981 (10.93)        5,521.3780 (15.22)        2,876.1775 (8.28)        411.3732 (56.59)        2,762.2810 (7.97)         68.5747 (9.62)        22;43    347.6837 (0.12)        365           1
test_lz4[geo.protodata-cramjam]                          744.4300 (3.03)         1,499.9070 (4.14)           799.9747 (2.30)         82.7017 (11.38)          779.4825 (2.25)         27.9890 (3.93)        58;87  1,250.0395 (0.43)       1174           1
test_lz4[geo.protodata-python-lz4]                       694.2170 (2.83)         1,546.8430 (4.26)           722.2650 (2.08)         47.9471 (6.60)           718.2080 (2.07)         22.3070 (3.13)        51;71  1,384.5333 (0.48)       1383           1
test_lz4[html-cramjam]                                   778.6801 (3.17)         1,917.2900 (5.29)           832.4302 (2.40)        111.0058 (15.27)          805.6900 (2.32)         13.8602 (1.95)       58;284  1,201.3020 (0.42)       1162           1
test_lz4[html-python-lz4]                                716.7751 (2.92)         1,543.9009 (4.26)           757.5458 (2.18)        105.6609 (14.53)          730.4699 (2.11)         27.0145 (3.79)       63;104  1,320.0522 (0.46)       1340           1
test_lz4[html_x_4-cramjam]                             3,105.4450 (12.66)        4,875.9540 (13.44)        3,301.6318 (9.51)        241.6346 (33.24)        3,248.0890 (9.37)        129.7960 (18.21)       20;21    302.8805 (0.11)        289           1
test_lz4[html_x_4-python-lz4]                          3,016.9110 (12.30)        5,823.1900 (16.06)        3,218.6525 (9.27)        294.3938 (40.49)        3,157.1820 (9.11)        103.9958 (14.59)       20;23    310.6890 (0.11)        255           1
test_lz4[kppkn.gtb-cramjam]                            3,345.9300 (13.64)        5,890.7979 (16.24)        3,538.4708 (10.19)       358.4893 (49.31)        3,434.2399 (9.91)        107.9319 (15.15)       21;28    282.6080 (0.10)        286           1
test_lz4[kppkn.gtb-python-lz4]                         3,323.8300 (13.55)        6,563.6940 (18.10)        3,649.2799 (10.51)       507.6818 (69.83)        3,453.0250 (9.96)        141.2800 (19.83)       26;48    274.0267 (0.10)        274           1
test_lz4[lcet10.txt-cramjam]                           7,444.1610 (30.34)       13,636.0540 (37.60)        8,044.9605 (23.17)     1,071.5044 (147.39)       7,670.1291 (22.13)       302.9506 (42.51)       15;22    124.3014 (0.04)        129           1
test_lz4[lcet10.txt-python-lz4]                        8,787.4030 (35.82)       15,124.7031 (41.70)        9,523.0310 (27.42)     1,039.6363 (143.00)       9,116.1070 (26.30)       404.7097 (56.79)       15;20    105.0086 (0.04)        113           1
test_lz4[paper-100k.pdf-cramjam]                       1,165.2170 (4.75)         3,391.3370 (9.35)         1,238.3142 (3.57)        187.5870 (25.80)        1,197.4330 (3.45)         45.5229 (6.39)        41;60    807.5495 (0.28)        806           1
test_lz4[paper-100k.pdf-python-lz4]                    1,786.4701 (7.28)         3,989.5640 (11.00)        1,930.3730 (5.56)        270.5717 (37.22)        1,852.4720 (5.34)         43.4450 (6.10)        38;75    518.0346 (0.18)        533           1
test_lz4[plrabn12.txt-cramjam]                        10,025.1100 (40.86)       15,823.3760 (43.63)       11,022.9072 (31.74)     1,388.2368 (190.95)      10,650.3405 (30.73)       757.5119 (106.30)        7;7     90.7202 (0.03)         62           1
test_lz4[plrabn12.txt-python-lz4]                     11,853.4301 (48.32)       17,182.6570 (47.37)       13,388.9093 (38.55)     1,159.1757 (159.45)      12,978.2920 (37.44)     1,572.6237 (220.69)       18;1     74.6887 (0.03)         77           1
test_lz4[urls.10K-cramjam]                             8,241.6250 (33.59)       15,744.7390 (43.41)        8,967.4984 (25.82)     1,247.4730 (171.59)       8,477.2279 (24.46)       717.1374 (100.64)        9;9    111.5138 (0.04)        104           1
test_lz4[urls.10K-python-lz4]                         10,320.9730 (42.07)       33,072.6539 (91.18)       14,466.2239 (41.66)     4,628.6794 (636.68)      12,947.5440 (37.35)     4,977.0970 (698.44)        4;4     69.1265 (0.02)         94           1
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```

#### LZ4 _block_ format

`make bench-lz4-block`

```bash
---------------------------------------------------------------------------------------------------------------- benchmark: 28 tests -----------------------------------------------------------------------------------------------------------------
Name (time in us)                                                 Min                     Max                    Mean                StdDev                  Median                 IQR            Outliers          OPS            Rounds  Iterations
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_lz4_block[Mark.Twain-Tom.Sawyer.txt-cramjam]             40.9340 (1.37)         184.3430 (1.19)          42.9931 (1.35)         4.6601 (1.0)           41.5510 (1.37)       0.3530 (1.92)    1034;1427  23,259.5474 (0.74)       9422           1
test_lz4_block[Mark.Twain-Tom.Sawyer.txt-python-lz4]          38.4370 (1.28)         154.2830 (1.0)           40.6069 (1.28)         5.5279 (1.19)          38.9110 (1.28)       0.4095 (2.23)    1107;2431  24,626.3528 (0.78)      10544           1
test_lz4_block[alice29.txt-cramjam]                          472.0840 (15.77)        713.6530 (4.63)         494.8957 (15.56)       25.8485 (5.55)         493.4380 (16.27)     31.5625 (171.54)     190;53   2,020.6276 (0.06)       1839           1
test_lz4_block[alice29.txt-python-lz4]                       513.4790 (17.15)        857.2650 (5.56)         537.8642 (16.91)       27.2345 (5.84)         535.3320 (17.65)     34.5312 (187.68)     215;47   1,859.2053 (0.06)       1863           1
test_lz4_block[asyoulik.txt-cramjam]                         380.3780 (12.71)        635.7260 (4.12)         401.1980 (12.61)       28.4094 (6.10)         393.5930 (12.98)     23.0175 (125.10)    193;141   2,492.5350 (0.08)       2096           1
test_lz4_block[asyoulik.txt-python-lz4]                      416.9540 (13.93)        734.8450 (4.76)         437.0448 (13.74)       23.5893 (5.06)         437.7690 (14.44)     24.1480 (131.24)    251;108   2,288.0950 (0.07)       2290           1
test_lz4_block[fifty-four-mb-random-cramjam]              19,617.3530 (655.40)    20,615.6860 (133.62)    19,903.2510 (625.72)     196.4017 (42.15)     19,867.1210 (655.12)   183.0970 (995.12)       14;3      50.2430 (0.00)         50           1
test_lz4_block[fifty-four-mb-random-python-lz4]          115,706.2050 (>1000.0)  116,848.0440 (757.36)   116,398.5400 (>1000.0)    421.1026 (90.36)    116,555.1350 (>1000.0)  572.2738 (>1000.0)       3;0       8.5912 (0.00)          9           1
test_lz4_block[fifty-four-mb-repeating-cramjam]           16,384.0190 (547.37)    34,818.2670 (225.68)    17,412.4441 (547.42)   3,003.0026 (644.41)    16,854.9620 (555.79)   258.3560 (>1000.0)       1;4      57.4302 (0.00)         36           1
test_lz4_block[fifty-four-mb-repeating-python-lz4]        58,464.7160 (>1000.0)   61,384.1530 (397.87)    59,202.9512 (>1000.0)    776.3388 (166.59)    59,151.1860 (>1000.0)  644.4060 (>1000.0)       2;2      16.8910 (0.00)         17           1
test_lz4_block[fireworks.jpeg-cramjam]                        29.9320 (1.0)          161.8750 (1.05)          31.8084 (1.0)          5.0188 (1.08)          30.3260 (1.0)        0.1840 (1.0)     1946;3770  31,438.2794 (1.0)       18014           1
test_lz4_block[fireworks.jpeg-python-lz4]                     34.4750 (1.15)         169.8460 (1.10)          36.7064 (1.15)         5.2782 (1.13)          35.1650 (1.16)       0.8880 (4.83)    1940;2096  27,243.1909 (0.87)      17242           1
test_lz4_block[geo.protodata-cramjam]                        140.1360 (4.68)         291.7290 (1.89)         147.5534 (4.64)        12.0341 (2.58)         141.8530 (4.68)       9.3140 (50.62)     990;375   6,777.2052 (0.22)       5938           1
test_lz4_block[geo.protodata-python-lz4]                     144.4831 (4.83)         350.4770 (2.27)         152.2133 (4.79)        12.8940 (2.77)         146.0120 (4.81)      10.0460 (54.60)    1043;347   6,569.7283 (0.21)       5972           1
test_lz4_block[html-cramjam]                                 142.9390 (4.78)         277.8630 (1.80)         149.9061 (4.71)        11.7107 (2.51)         144.4170 (4.76)       9.1133 (49.53)     846;321   6,670.8442 (0.21)       5319           1
test_lz4_block[html-python-lz4]                              147.3250 (4.92)         279.1871 (1.81)         154.4610 (4.86)        11.5615 (2.48)         149.0470 (4.91)       9.1137 (49.53)     994;375   6,474.1270 (0.21)       6013           1
test_lz4_block[html_x_4-cramjam]                             600.2710 (20.05)        941.8120 (6.10)         628.8270 (19.77)       29.0365 (6.23)         623.0760 (20.55)     39.2540 (213.34)     186;28   1,590.2625 (0.05)       1491           1
test_lz4_block[html_x_4-python-lz4]                          625.6220 (20.90)        930.9070 (6.03)         655.2924 (20.60)       30.3427 (6.51)         648.3260 (21.38)     39.3870 (214.07)     196;40   1,526.0362 (0.05)       1448           1
test_lz4_block[kppkn.gtb-cramjam]                            446.5680 (14.92)        729.3510 (4.73)         466.9985 (14.68)       24.7114 (5.30)         467.1435 (15.40)     27.8070 (151.13)     217;64   2,141.3345 (0.07)       2110           1
test_lz4_block[kppkn.gtb-python-lz4]                         457.6420 (15.29)        673.1950 (4.36)         477.8109 (15.02)       23.3480 (5.01)         473.5590 (15.62)     26.9885 (146.68)     239;72   2,092.8781 (0.07)       2073           1
test_lz4_block[lcet10.txt-cramjam]                         1,300.3600 (43.44)      1,639.6500 (10.63)      1,358.7622 (42.72)       44.2987 (9.51)       1,350.1540 (44.52)     57.7395 (313.81)     228;11     735.9640 (0.02)        720           1
test_lz4_block[lcet10.txt-python-lz4]                      1,410.9100 (47.14)      1,763.4770 (11.43)      1,474.0569 (46.34)       48.0336 (10.31)      1,465.7810 (48.33)     60.8988 (330.98)     176;13     678.3999 (0.02)        621           1
test_lz4_block[paper-100k.pdf-cramjam]                        75.7430 (2.53)         213.3430 (1.38)          80.0105 (2.52)         8.9087 (1.91)          76.5580 (2.52)       2.2090 (12.01)    848;1656  12,498.3609 (0.40)       8862           1
test_lz4_block[paper-100k.pdf-python-lz4]                     82.7410 (2.76)         217.1710 (1.41)          87.4508 (2.75)         9.4264 (2.02)          83.8570 (2.77)       0.8590 (4.67)     905;2221  11,435.0004 (0.36)       9246           1
test_lz4_block[plrabn12.txt-cramjam]                       1,527.4300 (51.03)      1,917.3070 (12.43)      1,594.6245 (50.13)       55.9814 (12.01)      1,584.3080 (52.24)     64.3090 (349.52)     125;16     627.1069 (0.02)        511           1
test_lz4_block[plrabn12.txt-python-lz4]                    2,011.2020 (67.19)      2,540.4010 (16.47)      2,123.5693 (66.76)       93.4244 (20.05)      2,098.7020 (69.20)    111.1333 (604.00)     105;22     470.9053 (0.01)        459           1
test_lz4_block[urls.10K-cramjam]                           1,511.1370 (50.49)      2,008.8290 (13.02)      1,591.5999 (50.04)       65.2772 (14.01)      1,578.4930 (52.05)     61.4250 (333.84)     120;29     628.2986 (0.02)        511           1
test_lz4_block[urls.10K-python-lz4]                        2,001.9611 (66.88)      2,490.6120 (16.14)      2,103.2583 (66.12)       66.4167 (14.25)      2,090.2720 (68.93)     78.7600 (428.06)     111;12     475.4528 (0.02)        427           1
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```

#### Brotli

`make bench-brotli`

```bash
------------------------------------------------------------------------------------------------- benchmark: 24 tests --------------------------------------------------------------------------------------------------
Name (time in ms)                                       Min                 Max                Mean             StdDev              Median                IQR            Outliers      OPS            Rounds  Iterations
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_brotli[Mark.Twain-Tom.Sawyer.txt-brotli]       10.9184 (1.0)       14.6035 (1.0)       12.3503 (1.0)       1.1312 (1.74)      11.8966 (1.0)       2.1789 (5.62)         30;0  80.9697 (1.0)          82           1
test_brotli[Mark.Twain-Tom.Sawyer.txt-cramjam]      14.0977 (1.29)      17.0087 (1.16)      14.5718 (1.18)      0.6504 (1.0)       14.3074 (1.20)      0.3876 (1.0)           5;5  68.6259 (0.85)         47           1
test_brotli[alice29.txt-brotli]                    141.2160 (12.93)    145.2801 (9.95)     142.9074 (11.57)     1.4637 (2.25)     142.5277 (11.98)     2.1899 (5.65)          2;0   6.9975 (0.09)          7           1
test_brotli[alice29.txt-cramjam]                   152.6566 (13.98)    158.3756 (10.85)    154.4223 (12.50)     1.9262 (2.96)     154.1212 (12.96)     1.7533 (4.52)          1;1   6.4757 (0.08)          7           1
test_brotli[asyoulik.txt-brotli]                   110.8312 (10.15)    116.3864 (7.97)     113.1018 (9.16)      1.9668 (3.02)     112.1098 (9.42)      3.2754 (8.45)          2;0   8.8416 (0.11)          9           1
test_brotli[asyoulik.txt-cramjam]                  120.2650 (11.01)    142.7060 (9.77)     130.4795 (10.56)     9.0843 (13.97)    129.5092 (10.89)    16.0443 (41.40)         3;0   7.6640 (0.09)          9           1
test_brotli[fireworks.jpeg-brotli]                  35.0424 (3.21)      52.9552 (3.63)      42.1037 (3.41)      6.8887 (10.59)     37.3856 (3.14)     12.9743 (33.48)         8;0  23.7509 (0.29)         23           1
test_brotli[fireworks.jpeg-cramjam]                 45.6972 (4.19)      51.4216 (3.52)      47.9267 (3.88)      1.4601 (2.25)      48.0562 (4.04)      2.0533 (5.30)          7;0  20.8652 (0.26)         22           1
test_brotli[geo.protodata-brotli]                   70.8891 (6.49)      73.5486 (5.04)      72.2200 (5.85)      0.8960 (1.38)      72.7037 (6.11)      1.5739 (4.06)          5;0  13.8466 (0.17)         14           1
test_brotli[geo.protodata-cramjam]                  91.4206 (8.37)      94.7283 (6.49)      93.2170 (7.55)      0.9418 (1.45)      93.4534 (7.86)      1.1083 (2.86)          4;0  10.7277 (0.13)         11           1
test_brotli[html-brotli]                            74.1237 (6.79)      76.8266 (5.26)      75.6515 (6.13)      0.9074 (1.40)      75.8785 (6.38)      1.3270 (3.42)          5;0  13.2185 (0.16)         13           1
test_brotli[html-cramjam]                          102.4915 (9.39)     125.3486 (8.58)     112.2085 (9.09)      9.9984 (15.37)    106.6630 (8.97)     19.7193 (50.88)         3;0   8.9120 (0.11)         10           1
test_brotli[html_x_4-brotli]                        87.5360 (8.02)     114.3479 (7.83)      97.7994 (7.92)     11.4060 (17.54)     90.7073 (7.62)     21.9963 (56.75)         3;0  10.2250 (0.13)         10           1
test_brotli[html_x_4-cramjam]                      105.3706 (9.65)     111.3189 (7.62)     107.3742 (8.69)      1.9368 (2.98)     107.6139 (9.05)      2.9738 (7.67)          3;0   9.3132 (0.12)         10           1
test_brotli[kppkn.gtb-brotli]                      271.9317 (24.91)    322.6200 (22.09)    284.3380 (23.02)    21.5044 (33.07)    276.2475 (23.22)    15.4194 (39.78)         1;1   3.5169 (0.04)          5           1
test_brotli[kppkn.gtb-cramjam]                     323.5000 (29.63)    328.4267 (22.49)    324.7345 (26.29)     2.0891 (3.21)     324.0014 (27.23)     1.7883 (4.61)          1;1   3.0794 (0.04)          5           1
test_brotli[lcet10.txt-brotli]                     442.0427 (40.49)    504.8650 (34.57)    462.6317 (37.46)    27.2407 (41.89)    446.6555 (37.54)    39.1690 (101.06)        1;0   2.1615 (0.03)          5           1
test_brotli[lcet10.txt-cramjam]                    444.3975 (40.70)    511.8355 (35.05)    463.9231 (37.56)    27.3992 (42.13)    453.0632 (38.08)    24.6888 (63.70)         1;1   2.1555 (0.03)          5           1
test_brotli[paper-100k.pdf-brotli]                 262.9858 (24.09)    307.7572 (21.07)    276.0070 (22.35)    18.9312 (29.11)    265.8558 (22.35)    22.6216 (58.37)         1;0   3.6231 (0.04)          5           1
test_brotli[paper-100k.pdf-cramjam]                 91.9629 (8.42)      95.6179 (6.55)      93.8155 (7.60)      1.3233 (2.03)      93.5711 (7.87)      2.4300 (6.27)          5;0  10.6592 (0.13)         11           1
test_brotli[plrabn12.txt-brotli]                   483.2054 (44.26)    497.8085 (34.09)    487.2513 (39.45)     6.0998 (9.38)     484.6980 (40.74)     6.3864 (16.48)         1;0   2.0523 (0.03)          5           1
test_brotli[plrabn12.txt-cramjam]                  502.6484 (46.04)    530.4874 (36.33)    511.1903 (41.39)    11.2501 (17.30)    506.6601 (42.59)    11.8233 (30.51)         1;0   1.9562 (0.02)          5           1
test_brotli[urls.10K-brotli]                       731.0115 (66.95)    808.1545 (55.34)    753.5258 (61.01)    32.1086 (49.37)    736.5432 (61.91)    35.3638 (91.24)         1;0   1.3271 (0.02)          5           1
test_brotli[urls.10K-cramjam]                      722.6436 (66.19)    823.0498 (56.36)    743.6850 (60.22)    44.3756 (68.23)    724.4152 (60.89)    26.3356 (67.95)         1;1   1.3447 (0.02)          5           1
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```

#### Bzip2

`make bench-bzip2`

```bash
-------------------------------------------------------------------------------------------------------- benchmark: 28 tests ---------------------------------------------------------------------------------------------------------
Name (time in ms)                                         Min                    Max                   Mean              StdDev                 Median                 IQR            Outliers       OPS            Rounds  Iterations
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_bzip2[Mark.Twain-Tom.Sawyer.txt-bzip2]            2.1986 (1.03)          3.4193 (1.0)           2.3997 (1.00)       0.1607 (1.0)           2.3601 (1.02)       0.1701 (1.0)         68;14  416.7229 (1.00)        397           1
test_bzip2[Mark.Twain-Tom.Sawyer.txt-cramjam]          2.1309 (1.0)           3.4771 (1.02)          2.3924 (1.0)        0.2410 (1.50)          2.3114 (1.0)        0.2180 (1.28)        52;28  417.9866 (1.0)         392           1
test_bzip2[alice29.txt-bzip2]                         16.0991 (7.56)         17.7243 (5.18)         16.5891 (6.93)       0.3358 (2.09)         16.5265 (7.15)       0.3215 (1.89)         10;3   60.2804 (0.14)         43           1
test_bzip2[alice29.txt-cramjam]                       16.1838 (7.59)         17.7832 (5.20)         16.8158 (7.03)       0.2674 (1.66)         16.8085 (7.27)       0.2406 (1.41)         13;5   59.4677 (0.14)         58           1
test_bzip2[asyoulik.txt-bzip2]                        13.0441 (6.12)         15.9405 (4.66)         13.5905 (5.68)       0.3904 (2.43)         13.5189 (5.85)       0.3070 (1.80)          6;2   73.5809 (0.18)         68           1
test_bzip2[asyoulik.txt-cramjam]                      13.4303 (6.30)         15.0016 (4.39)         13.8275 (5.78)       0.2678 (1.67)         13.7867 (5.96)       0.2544 (1.50)         13;4   72.3196 (0.17)         71           1
test_bzip2[fifty-four-mb-random-bzip2]            10,386.9692 (>1000.0)  10,489.5419 (>1000.0)  10,418.5051 (>1000.0)   41.3389 (257.18)   10,408.8554 (>1000.0)   42.6205 (250.57)        1;0    0.0960 (0.00)          5           1
test_bzip2[fifty-four-mb-random-cramjam]          10,362.6044 (>1000.0)  11,798.0335 (>1000.0)  10,678.7979 (>1000.0)  626.7239 (>1000.0)  10,417.7132 (>1000.0)  421.0702 (>1000.0)       1;1    0.0936 (0.00)          5           1
test_bzip2[fifty-four-mb-repeating-bzip2]         15,823.1287 (>1000.0)  15,906.3119 (>1000.0)  15,877.7192 (>1000.0)   33.5349 (208.63)   15,880.1587 (>1000.0)   42.4617 (249.63)        1;0    0.0630 (0.00)          5           1
test_bzip2[fifty-four-mb-repeating-cramjam]       14,768.4424 (>1000.0)  15,670.1805 (>1000.0)  15,056.9177 (>1000.0)  365.0722 (>1000.0)  14,876.7961 (>1000.0)  411.8626 (>1000.0)       1;0    0.0664 (0.00)          5           1
test_bzip2[fireworks.jpeg-bzip2]                      19.6575 (9.23)         26.6222 (7.79)         20.5149 (8.57)       1.0187 (6.34)         20.3994 (8.83)       0.6566 (3.86)          3;1   48.7451 (0.12)         48           1
test_bzip2[fireworks.jpeg-cramjam]                    20.7419 (9.73)         23.1552 (6.77)         21.5242 (9.00)       0.4398 (2.74)         21.4853 (9.30)       0.5220 (3.07)          9;2   46.4594 (0.11)         46           1
test_bzip2[geo.protodata-bzip2]                       13.7524 (6.45)         16.2448 (4.75)         14.3407 (5.99)       0.4783 (2.98)         14.2251 (6.15)       0.4630 (2.72)         13;5   69.7315 (0.17)         71           1
test_bzip2[geo.protodata-cramjam]                     14.2087 (6.67)         15.8775 (4.64)         14.5802 (6.09)       0.2828 (1.76)         14.5270 (6.29)       0.2686 (1.58)         10;3   68.5860 (0.16)         59           1
test_bzip2[html-bzip2]                                10.0543 (4.72)         11.7887 (3.45)         10.5992 (4.43)       0.3381 (2.10)         10.5395 (4.56)       0.4206 (2.47)         25;3   94.3465 (0.23)         93           1
test_bzip2[html-cramjam]                              10.2327 (4.80)         15.4595 (4.52)         11.1203 (4.65)       1.1808 (7.35)         10.6782 (4.62)       0.5143 (3.02)         9;11   89.9255 (0.22)         93           1
test_bzip2[html_x_4-bzip2]                            54.2398 (25.45)        56.9572 (16.66)        55.2108 (23.08)      0.7869 (4.90)         55.2028 (23.88)      1.1302 (6.64)          5;0   18.1124 (0.04)         17           1
test_bzip2[html_x_4-cramjam]                          55.1079 (25.86)        57.9107 (16.94)        56.1745 (23.48)      0.9576 (5.96)         55.8812 (24.18)      1.7999 (10.58)         9;0   17.8017 (0.04)         18           1
test_bzip2[kppkn.gtb-bzip2]                           14.9813 (7.03)         21.7189 (6.35)         15.8015 (6.60)       0.8125 (5.06)         15.6806 (6.78)       0.3582 (2.11)          3;3   63.2852 (0.15)         65           1
test_bzip2[kppkn.gtb-cramjam]                         15.1289 (7.10)         16.4051 (4.80)         15.6411 (6.54)       0.2986 (1.86)         15.6152 (6.76)       0.4326 (2.54)         21;0   63.9343 (0.15)         62           1
test_bzip2[lcet10.txt-bzip2]                          44.8801 (21.06)        47.0429 (13.76)        45.7175 (19.11)      0.5902 (3.67)         45.5279 (19.70)      0.7627 (4.48)          6;0   21.8735 (0.05)         22           1
test_bzip2[lcet10.txt-cramjam]                        44.3920 (20.83)        46.8710 (13.71)        45.5553 (19.04)      0.6376 (3.97)         45.5037 (19.69)      0.7831 (4.60)          5;0   21.9513 (0.05)         19           1
test_bzip2[paper-100k.pdf-bzip2]                      14.6436 (6.87)         17.0778 (4.99)         15.2104 (6.36)       0.3691 (2.30)         15.1629 (6.56)       0.3812 (2.24)         11;2   65.7443 (0.16)         66           1
test_bzip2[paper-100k.pdf-cramjam]                    15.4920 (7.27)         17.5886 (5.14)         15.9745 (6.68)       0.3879 (2.41)         15.9001 (6.88)       0.3893 (2.29)          9;4   62.5996 (0.15)         62           1
test_bzip2[plrabn12.txt-bzip2]                        55.4209 (26.01)        59.1872 (17.31)        56.8947 (23.78)      1.0537 (6.56)         56.7556 (24.55)      1.3448 (7.91)          6;0   17.5763 (0.04)         18           1
test_bzip2[plrabn12.txt-cramjam]                      55.9271 (26.25)        58.9951 (17.25)        56.8192 (23.75)      0.8190 (5.10)         56.6420 (24.51)      1.3035 (7.66)          5;0   17.5997 (0.04)         18           1
test_bzip2[urls.10K-bzip2]                            81.4232 (38.21)       111.9462 (32.74)        85.9987 (35.95)      9.1141 (56.70)        82.2004 (35.56)      2.4570 (14.44)         1;2   11.6281 (0.03)         11           1
test_bzip2[urls.10K-cramjam]                          80.2907 (37.68)        85.2344 (24.93)        82.5610 (34.51)      1.4309 (8.90)         82.6907 (35.78)      2.0479 (12.04)         5;0   12.1123 (0.03)         13           1
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```


#### ZSTD

`make bench-zstd`

```bash 
------------------------------------------------------------------------------------------------------------- benchmark: 28 tests -------------------------------------------------------------------------------------------------------------
Name (time in us)                                         Min                     Max                    Mean                StdDev                  Median                   IQR            Outliers         OPS            Rounds  Iterations
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
test_zstd[Mark.Twain-Tom.Sawyer.txt-cramjam]         162.9620 (1.09)         406.7180 (1.0)          176.6457 (1.0)         18.1423 (1.14)         169.1270 (1.0)         10.4961 (30.47)     201;206  5,661.0504 (1.0)        1524           1
test_zstd[Mark.Twain-Tom.Sawyer.txt-zstd]            150.1160 (1.0)          476.3250 (1.17)         254.5834 (1.44)        73.5374 (4.64)         286.3150 (1.69)       111.8249 (324.63)       15;1  3,927.9858 (0.69)         50           1
test_zstd[alice29.txt-cramjam]                     1,375.7630 (9.16)       1,755.1420 (4.32)       1,409.3451 (7.98)        44.2825 (2.79)       1,399.8351 (8.28)        39.0127 (113.25)      63;29    709.5494 (0.13)        675           1
test_zstd[alice29.txt-zstd]                        1,325.4730 (8.83)       2,010.5341 (4.94)       1,360.4852 (7.70)        62.9252 (3.97)       1,345.8120 (7.96)        39.4245 (114.45)      35;29    735.0319 (0.13)        691           1
test_zstd[asyoulik.txt-cramjam]                    1,215.3659 (8.10)       1,828.1790 (4.49)       1,250.6696 (7.08)        57.2588 (3.61)       1,238.3070 (7.32)        38.2224 (110.96)      53;47    799.5717 (0.14)        759           1
test_zstd[asyoulik.txt-zstd]                       1,159.6669 (7.73)       1,858.6870 (4.57)       1,189.6754 (6.73)        54.2058 (3.42)       1,175.6300 (6.95)        34.5710 (100.36)      54;39    840.5654 (0.15)        816           1
test_zstd[fifty-four-mb-random-cramjam]          126,119.5041 (840.15)   127,714.4219 (314.01)   126,973.2383 (718.80)     588.7249 (37.11)    127,016.0485 (751.01)     922.4786 (>1000.0)       4;0      7.8757 (0.00)          8           1
test_zstd[fifty-four-mb-random-zstd]              54,271.3830 (361.53)    67,254.4950 (165.36)    62,470.3614 (353.65)   3,707.2937 (233.68)    63,691.2219 (376.59)   4,924.4625 (>1000.0)       4;0     16.0076 (0.00)         16           1
test_zstd[fifty-four-mb-repeating-cramjam]        49,074.5769 (326.91)    50,540.2781 (124.26)    49,350.9156 (279.38)     374.5157 (23.61)     49,230.9110 (291.09)     269.6409 (782.76)        1;1     20.2630 (0.00)         14           1
test_zstd[fifty-four-mb-repeating-zstd]           27,456.1089 (182.90)    30,666.1710 (75.40)     28,946.7121 (163.87)     620.6539 (39.12)     29,004.8160 (171.50)     532.4374 (>1000.0)       8;4     34.5462 (0.01)         33           1
test_zstd[fireworks.jpeg-cramjam]                    287.1940 (1.91)         503.2960 (1.24)         296.2061 (1.68)        19.8144 (1.25)         288.1080 (1.70)         8.4531 (24.54)     228;298  3,376.0276 (0.60)       2354           1
test_zstd[fireworks.jpeg-zstd]                       216.6729 (1.44)         594.6030 (1.46)         221.7168 (1.26)        16.1432 (1.02)         217.1519 (1.28)         0.3445 (1.0)       280;881  4,510.2582 (0.80)       4208           1
test_zstd[geo.protodata-cramjam]                     327.6070 (2.18)         517.6039 (1.27)         338.9284 (1.92)        17.7418 (1.12)         332.8320 (1.97)         8.2538 (23.96)     259;399  2,950.4755 (0.52)       2635           1
test_zstd[geo.protodata-zstd]                        280.3099 (1.87)         669.7009 (1.65)         289.7946 (1.64)        15.8649 (1.0)          285.2400 (1.69)         2.6374 (7.66)      288;645  3,450.7199 (0.61)       3200           1
test_zstd[html-cramjam]                              356.0591 (2.37)         772.1799 (1.90)         366.8109 (2.08)        23.6813 (1.49)         359.2240 (2.12)         5.6227 (16.32)     175;413  2,726.2002 (0.48)       2443           1
test_zstd[html-zstd]                                 312.1750 (2.08)         533.3930 (1.31)         321.4469 (1.82)        19.8513 (1.25)         314.5090 (1.86)         3.7401 (10.86)     246;582  3,110.9334 (0.55)       2934           1
test_zstd[html_x_4-cramjam]                          476.6970 (3.18)         719.8390 (1.77)         490.4110 (2.78)        21.8491 (1.38)         481.2561 (2.85)        15.6217 (45.35)     192;152  2,039.1062 (0.36)       1797           1
test_zstd[html_x_4-zstd]                             364.6320 (2.43)         644.5120 (1.58)         374.6641 (2.12)        19.1959 (1.21)         367.3399 (2.17)         5.6954 (16.53)     231;447  2,669.0574 (0.47)       2477           1
test_zstd[kppkn.gtb-cramjam]                       1,093.0240 (7.28)       1,506.8919 (3.71)       1,131.5039 (6.41)        49.7900 (3.14)       1,117.8980 (6.61)        44.4521 (129.04)      92;52    883.7795 (0.16)        843           1
test_zstd[kppkn.gtb-zstd]                          1,044.6580 (6.96)       1,529.9510 (3.76)       1,069.5185 (6.05)        38.3409 (2.42)       1,053.7340 (6.23)        28.2210 (81.93)       77;49    935.0002 (0.17)        896           1
test_zstd[lcet10.txt-cramjam]                      3,546.4830 (23.62)      4,082.9000 (10.04)      3,626.4003 (20.53)       78.0370 (4.92)       3,605.1755 (21.32)       78.6935 (228.45)      31;11    275.7555 (0.05)        260           1
test_zstd[lcet10.txt-zstd]                         3,407.8470 (22.70)      4,383.9460 (10.78)      3,496.2894 (19.79)      105.5919 (6.66)       3,474.2125 (20.54)       70.1590 (203.67)      22;22    286.0175 (0.05)        286           1
test_zstd[paper-100k.pdf-cramjam]                    494.6779 (3.30)         958.8940 (2.36)         510.4041 (2.89)        32.5814 (2.05)         499.8320 (2.96)        13.1470 (38.17)      95;186  1,959.2320 (0.35)       1821           1
test_zstd[paper-100k.pdf-zstd]                       480.3210 (3.20)         801.3280 (1.97)         493.8572 (2.80)        26.0754 (1.64)         484.5071 (2.86)        14.3057 (41.53)     133;137  2,024.8767 (0.36)       1655           1
test_zstd[plrabn12.txt-cramjam]                    4,773.0900 (31.80)      5,415.0301 (13.31)      4,930.6590 (27.91)      134.2914 (8.46)       4,886.1710 (28.89)      140.8120 (408.78)      38;10    202.8126 (0.04)        165           1
test_zstd[plrabn12.txt-zstd]                       4,617.4621 (30.76)      5,391.8620 (13.26)      4,735.0956 (26.81)      101.1250 (6.37)       4,712.4890 (27.86)       77.8873 (226.11)      28;11    211.1890 (0.04)        203           1
test_zstd[urls.10K-cramjam]                        4,454.6180 (29.67)      6,157.7980 (15.14)      4,578.2559 (25.92)      153.7447 (9.69)       4,547.1541 (26.89)       93.7060 (272.03)      12;11    218.4238 (0.04)        194           1
test_zstd[urls.10K-zstd]                           4,201.4410 (27.99)      4,969.7730 (12.22)      4,298.6442 (24.33)      103.9367 (6.55)       4,271.2825 (25.25)       69.8585 (202.80)      18;13    232.6315 (0.04)        228           1
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
```
